%% Statistical processing and analysis of surface topography data for a machine-based evaluation of the measurement uncertainty
%%% Matteo Gilardi %%%

clear
clc
close all

%% Data Import
lp=1;
ns=0;
lCount=0;
dCount=0;
aCount=0;
tCount=0;
nLoop=0;

while lp

    [nFile,fPath,~]=uigetfile({'*.*';'*.lext';'*.datx';'*.asc';'*.txt'}, ...
        'Select Source File/Files (.lext, .datx, .asc, .txt)','MultiSelect','on');
    nLoop=nLoop+1;

    if isequal(nFile,0)
        disp('The operation was canceled by the user.');
        lp=0;
    else
        if ~iscell(nFile), nFile={nFile}; end

        if nLoop==1
            C = cell(length(nFile),1);
            fileInput=struct('lext',C,'datx',C,'asc',C,'txt',C);
            S=struct('XIn',[],'YIn',[],'ZIn',[],'Zpc',[],'Zv',[],'Z',[],...
                'parameters',[],'Ix',[],'Iy',[]);
            data=struct('lext',S,'datx',S,'asc',S,'txt',S);
        end

        for q=1:length(nFile)
            inFile=nFile{q};
            fFilePath=fullfile(fPath,inFile);
            [~,~,extT]=fileparts(fFilePath);

            switch extT
                case '.lext'
                    lCount=lCount+1;
                    fileInput(lCount).lext=fFilePath;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Add here function for opening .lext file %%%%%
%%%%% lCount to control the files to be opened %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                case '.datx'
                    dCount=dCount+1;
                    fileInput(dCount).datx=fFilePath;
                    % dCount to control the files to be opened
                    [data.datx.XIn,data.datx.YIn,data.datx.ZIn(:,:,dCount),data.datx.Zv(:,:,dCount),...
                        data.datx.parameters,data.datx.Ix,data.datx.Iy] = datx_import(fFilePath);
                    data.datx.Zv=logical(data.datx.Zv);
                case '.asc'
                    aCount=aCount+1;
                    fileInput(aCount).asc=fFilePath;
                    % aCount to control the files to be opened
                    [data.asc.XIn,data.asc.YIn,data.asc.ZIn(:,:,aCount),data.asc.Zv(:,:,aCount),...
                        data.asc.parameters,data.asc.Ix,data.asc.Iy] = asc_import(fFilePath);
                    data.asc.Zv=logical(data.asc.Zv);
                case '.txt'
                    tCount=tCount+1;
                    fileInput(tCount).txt=fFilePath;
                    % tCount to control the files to be opened
                    [data.txt.XIn,data.txt.YIn,data.txt.ZIn(:,:,tCount),data.txt.Zv(:,:,tCount),...
                        data.txt.parameters,data.txt.Ix,data.txt.Iy] = txt_import(fFilePath);
                    data.txt.Zv=logical(data.txt.Zv);
               otherwise
                    ns=1;
                    clear fileInput
            end
        end
    if ns, fprintf('File(s) type not supported. \n'); ns=0; end

    if lp
        reply=input('Opening more file(s)? Y/N (enter --> Y):','s');
        if ~isempty(reply) && (reply=='N'||reply=='n')
        lp=0;
        end 
    end
    end
end 
                    
%% Delete not used variables and definition of the data structure
if lCount~=0,fprintf('Opening %d .lext file(s). \n', lCount);data=data.lext;end
if dCount~=0,fprintf('Opening %d .datx file(s). \n', dCount);data=data.datx;end
if aCount~=0,fprintf('Opening %d .asc file(s). \n', aCount);data=data.asc;end
if tCount~=0,fprintf('Opening %d .text file(s). \n', tCount);data=data.txt;end
if nLoop~=0, fprintf('Performed %d iterations. \n', nLoop); end
clear q extT ns fPath indx fFilePath C S inFile lp reply nLoop lCount dCount aCount tCount fileInput

%% Imported-Data Visualization 
t = '3D visualization of the imported surface';
visualization(data.XIn,data.YIn,data.ZIn,nFile,t)
clear t

%% Plane Correction
compTime=struct('planeCorr',[],'outMan',[],'linReg',[]);
reply=input('Applying the Plane Correction? Y/N (enter --> Y):','s');
if ~isempty(reply) && (reply=='N'||reply=='n')
    disp('The plane correction operation was canceled by the user.');
    data.Zpc=data.ZIn;
else
    [indx,tf] = pc_dialogbox();
    [data.Zpc,compTime.planeCorr] = plane_correction(indx,tf,data.XIn,data.YIn,data.ZIn);
end 
clear reply indx tf

%% Data Visualization after the plane correction
t = '3D visualization after the plane correction';
visualization(data.XIn,data.YIn,data.Zpc,nFile,t)
clear t

%% Voids management after the plane correction
data.Zpc(data.Zv)=0;

%% Roughness parameters before outliers management
S=struct('Sa',[],'Sq',[],'Sdq',[]);
rougPar=struct('BeforeOutMan',S,'AfterOutMan',S,'AfterLinReg',S);
[rougPar.BeforeOutMan.Sa,rougPar.BeforeOutMan.Sq,rougPar.BeforeOutMan.Sdq] = ...
    roughness_parameters(data.Zpc,data.parameters,data.Ix,data.Iy);
clear S

%% Spikes and Voids detection
spikes=struct('Xs',[],'Ys',[],'Zs',[]);
voids=struct('Xv',[],'Yv',[],'Zv',[]);
[spikes.Xs,spikes.Ys,spikes.Zs,voids.Xv,voids.Yv,voids.Zv] = ...
    sv_detection(data.Zpc,data.Zv,rougPar.BeforeOutMan.Sq,data.Ix,data.Iy);
clear S

%% Spikes and Voids identification on the surface
sv_visualization(nFile,data,spikes,voids)

%% Spikes management
[data.Zpc] = s_management(data.Zpc,rougPar.BeforeOutMan.Sq);

%% Data Visualization after the spikes management
t = '3D visualization after the spikes management';
visualization(data.XIn,data.YIn,data.Zpc,nFile,t)
clear t

%% Outliers Management
outliers=struct('Xol',[],'Yol',[],'Zol',[]);
reply=input('Managing the Outliers? Y/N (enter --> Y):','s');
if ~isempty(reply) && (reply=='N'||reply=='n')
    disp('The Outliers Management operation was canceled by the user.');
else
    [data.Zpc,zol,compTime.outMan] = chauvenet(data.Zpc);
%     [data.Zpc,zol,compTime.outMan] = modified_IQR(data.Zpc);
    [outliers.Xol,outliers.Yol,outliers.Zol] = ol_identification(data.Zpc,data.Ix,data.Iy,zol);
    ol_visualization(nFile,data,outliers);
end 
clear reply zol

%% Statistical Indexes
statIndx=struct('M',[],'Me',[],'V',[],'S',[]);
[statIndx.M,statIndx.Me,statIndx.V,statIndx.S] = stat_indx(data.Zpc);

%% Data Visualization after the outliers management
t = '3D visualization after the outliers management';
visualization(data.XIn,data.YIn,data.Zpc,nFile,t)
clear t

%% Roughness parameters after outliers management
[rougPar.AfterOutMan.Sa,rougPar.AfterOutMan.Sq,rougPar.AfterOutMan.Sdq] = ...
    roughness_parameters(data.Zpc,data.parameters,data.Ix,data.Iy);

%% Normality Test
NormTest_results=struct('data',[],'residuals',[]);
reply=input('Proceeding with the Normality Test for each pixel and then for the Residuals? Y/N (enter --> Y):','s');
if ~isempty(reply) && (reply=='N'||reply=='n')
    disp('The Normality Test operation was canceled by the user.');
else
    t='Normality Test for each pixel';
    [NormTest_results.data] = chi_squared_test(data.Zpc,t);
    % Normal Probability Plot
    npp(data.Zpc,NormTest_results.data);          
end    

%% Research of possible systematic behaviours
% data.Zpc=data.Zpc-statIndx.M;

% Upper triangular block covariance matrix
covUBM=covaMat_sampling_fcn(Zpc);

% Correlation length
[row,col,~]=size(covUBM);
N=row*col;
corr_dist=zeros(1,N);
parfor k=1:N
    [corr_dist(k),~]=correlation_distance(covUBM,0.5,k);
end
corr_dist=(reshape(corr_dist,row,col))';

% Regression
[~,~,pg]=size(Zpc);
t=-floor(pg/2):1:floor(pg/2)-1;
[Regression] = cov_linReg(data.Zpc,corr_dist,t);

clear k row col pg N

%% Significance Test
t = 'Residuals Normality Test';
NormTest_results.residuals = significance_test(Regression.Res,...
        Regression.RobustnessPar,t,reply);
clear t reply

%% Data Visualization after the correction of systematic behaviour
t = '3D visualization after the correction of systematic behaviour';
visualization(data.XIn,data.YIn,Regression.Corrected,nFile,t)
clear t

%% Roughness parameters after the correction of systematic behaviour
[rougPar.AfterLinReg.Sa,rougPar.AfterLinReg.Sq,rougPar.AfterLinReg.Sdq] = ...
    roughness_parameters(Regression.Corrected,data.parameters,data.Ix,data.Iy);

%% Uncertainty evaluation
uncertainty=struct('q',[],'U',[]);
[uncertainty.q,uncertainty.U] = uncertainty_evaluation(Regression);
clear S measInst

%% Uncertainty Visualization
U_visualization(uncertainty);
