%% Extraction of image parameters from the file

function [XIn,YIn,ZIn,Zv,parameters,Ix,Iy] = asc_import(pathfile)

%   column1: string (%s)
%	column2: string (%s)
%   column3: string (%s)
%	column4: double (%f)
formatSpec = '%s%s%s%f';                                                   
headerLines = 3;
lines = 10;
fileID = fopen(pathfile,'r');

if fileID == -1,disp('Error, check file name and path');
else
    dataArray = textscan(fileID, formatSpec,lines, 'HeaderLines', ...      
        headerLines, 'Delimiter', ' ');
    % It reads file data using the format of the data fields ('formatSpec')
    % for the number of 'lines'. In this specific case we have some 
    % header lines, and the delimiter is a white-space.
    fclose(fileID);
    parameters = dataArray{:,4};
end

% parameters(1) = forcecurve
% parameters(2) = x_pixels
% parameters(3) = y_pixels
% parameters(4) = x_length 
% parameters(5) = y_length 
% parameters(6) = x_offset
% parameters(7) = y_offset

Ix = parameters(4)/parameters(2); % Pixel's size, expressed in nm.
Iy = parameters(5)/parameters(3);

scala = 1e-3; % From nm to m.
% Definition of the grid in x-direction and y-direction.
x = [0.5*Ix:Ix:parameters(4)]*scala;                                       
y = [0.5*Iy:Iy:parameters(5)]*scala;
% XIn (In as input) is a matrix where each row is a copy of x, and YIn 
% is a matrix where each column is a copy of y. The grid represented
% by the coordinates X and Y has length(y) rows and length(x) columns.
[XIn,YIn] = meshgrid(x,y);                                                 

% Creation of a datastore from the collection of data specified by pathfile.
ttds = tabularTextDatastore(pathfile);
% Reading of all data in datastore; if all the data in the datastore 
% does not fit in memory, then readall returns an error.
table = readall(ttds); 

array = table2array(table);
array(:,parameters(2)+1) = [];
% Definition of the matrix with all the measurement data.
ZIn = array(1:1:parameters(3),1:1:parameters(2)).*scala;

%% Voids detection
% Whether in the image voids are present, their position is indicated
% in a logical table at the bottom of the input file.
if length(array) > parameters(3)
    % Definition of the matrix with all the voids.
    Zv = array(parameters(3)+1:1:2*parameters(3),...                       
        1:1:parameters(2));
else
    Zv=zeros(size(ZIn));
end
    
%% Value for Voids.
Zv = logical(Zv);
% The voids in the ZIn matrix are treated 'as not a number'
ZIn(Zv) = NaN;                                                             
