%% Chauvenet's criterion for each pixel

function [Zpc,zol,ct] = chauvenet(Zpc)

tic
[dimRow,dimCol,dimPage] = size(Zpc);
% Mean calculation along dimension 3 of the Zpc matrix, in this way
% the mean of each pixel of the Zpc value for the n images is obtained.
M = mean(Zpc,3);
% Standard deviation extraction along the dimension 3 of the Zpc matrix.
S = std(Zpc,0,3);
% Inverse cdf for the normal distribution with mean M and standard 
% deviation S. xli and xls are the exclusion boundaries.
xli = norminv(0.25/dimPage,M,S);                                             
xls = norminv(1-(0.25/dimPage),M,S);
% Chauvenet's criterion is applied to detect the outliers.
zol = Zpc > xls | Zpc < xli;
% This matrix represents which are the pixel where an outlier is detected. 
% The presence of an otlier is identified by the value 1.
n_ol = sum(zol,3);                                                         

figure('Name','Number of outliers per pixel-Chauvenet Criterion',...
    'NumberTitle','off');
surf(1:dimRow,1:dimCol,n_ol,'LineStyle','none','FaceColor',...
    'flat','EdgeColor','flat');
colormap jet
title(['Total number of Outliers: ',num2str(sum(n_ol,'all'))],...
    'FontSize',14,'FontName','Times New Roman')
xlabel('pixels in x-direction','FontSize',12,'FontName','Times New Roman');
ylabel('pixels in y-direction','FontSize',12,'FontName','Times New Roman');

% Identification of the non-outliers values using Chauvenet's criterion.
Znol = Zpc < xls | Zpc > xli;
% The mean is performed taking into account only the non-outliers values.
M = mean(reshape(Zpc(Znol),size(Zpc)),3);                                  
% The matrix with the mean value for each pixel is replicated along 
% the pages to have a matrix with the same size of Zpc.
M = repmat(M,[1 1 dimPage]);
% The outliers are replaced with the mean value. 
%The process is executed taking into account a single pixel each time.
Zpc(zol) = M(zol);                                                         
ct=toc;