%% Chi squared method for normality test

function [h] = chi_squared_test(Zpc,t)

[dimRow,dimCol,dimPage] = size(Zpc);
% Definition of the matrix with the Normal Distribution test result for each pixel.
h = zeros(dimRow,dimCol);                                                   
for i = 1:dimRow
    parfor j = 1:size(Zpc,2)
        % Reshape is used to get the column vector required by the function.
        % The column vector contains the repeated measurements for a pixel.
        h(i,j) = chi2gof(reshape(Zpc(i,j,:),dimPage,1),'Alpha',0.05);
        % If h=1, this indicates the rejection of the null hypothesis at the Alpha significance level.
        % If h=0, this indicates a failure to reject the null hypothesis at the Alpha significance level.
    end                                                                    
end                                                                         
                                                                           
s = sum(h,'all')/(dimRow*dimCol)*100;
format shortg
s = round(s,3,'significant');
figure('Name','Normality Test','NumberTitle','off');
title({t,['Percentage of pixels where the null hypothesis is rejected: ',...
    num2str(s),'%']},'FontSize',14,'FontName','Times New Roman')
surface(h,'LineStyle','none','FaceColor','flat')
axis square, axis equal, axis xy, axis([0 size(h,2) 0 size(h,1)])
xlabel('Pixels in x-direction','FontSize',12,'FontName','Times New Roman');
ylabel('Pixels in y-direction','FontSize',12,'FontName','Times New Roman');
legend('Null hypothesis failed to reject');

