%% Perform regression based on Covariance of pixels correlated 
% B is cell array containing the output of the regression.
% Regression is based on mvregress.

function [B] = cov_linReg(Zpc,l_matrix,x)

[dimRow,dimCol,~]=size(Zpc);

B.Par=zeros(dimRow,dimCol,2);
B.CovPar=cell(dimRow,dimCol);
B.Res=zeros(size(Zpc));
B.Corrected=zeros(size(Zpc));
B.Rsq = zeros(dimRow,dimCol);
B.rmse=zeros(dimRow,dimCol);
B.RobustnessPar=zeros(dimRow,dimCol,2);
B.se=zeros(dimRow,dimCol,2);

for w=1:dimRow
    for k=1:dimCol
       y=crop_corr_domain(Zpc,w,k,l_matrix(w,k));
       [nR,nC,~]=size(y);
       yt=reshape(permute(y,[2,1,3]),1,nR*nC,[]);
       % Arranged in [observation-by-variables].
       % the output is a column vector w by k.
       Y=permute(yt,[3,2,1]);
       [n,d]=size(Y);
       X = cell(n,1);
    for ii = 1:n
        X{ii} = [eye(d) x(ii)*eye(d)];
    end
    try 
        chol(cov(Y));
    catch ME
    
        if strcmp(ME.message,'Matrix must be positive definite.')
%             V=diag(diag(cov(Y)));
            [beta,~,E,SE] = mvregress(X,Y,'algorithm','cwls','covtype','full','varformat','beta');

        else
%             V=cov(Y);
            [beta,~,E,SE] = mvregress(X,Y,'algorithm','cwls','covar0',cov(Y),'covtype','full','varformat','beta');

        end
    end
    B.Par(w,k,1)=beta(w*k); % Intercept
    B.Par(w,k,2)=beta(w*k+d); % Slope
    B.CovPar{w,k}=[SE(w*k,w*k),SE(w*k,w*k+d);SE(w*k,w*k+d),SE(w*k+d,w*k+d)];
    B.Res(w,k,:)=E(:,w*k);
    B.Corrected(w,k,:)=mean(Y(:,w*k))+E(:,w*k);
    B.Rsq(w,k)=1-var(E(:,w*k))/var(Y(:,w*k));
    B.rmse(w,k)=std(E(:,w*k));
    B.chitest=chi2gof(E(:,w*k));
    U=tinv(0.975,n-2).*reshape(sqrt(diag(B.CovPar{w,k})),1,2);
    B.RobustnessPar(w,k,1)=(abs(B.Par(w,k,1))./U(1))>1;
    B.RobustnessPar(w,k,2)=(abs(B.Par(w,k,2))./U(2))>1;
    B.se(w,k,1)=B.CovPar{w,k}(1,1);
    B.se(w,k,2)=B.CovPar{w,k}(2,2);
    end
end
end

function [A]=crop_corr_domain(Zpc,i,j,l)
    [dimRow,dimCol,~]=size(Zpc);
    if i <= l+1
%             for j=1:dimCol
                if j <= l+1
                    A = Zpc(1:(i+l),j:(j+l),:);
                elseif j>= dimCol-l
                    A = Zpc(1:(i+l),(j-l):dimCol,:);
                else
                    A = Zpc(1:(i+l),(j-l):(j+l),:);
                end
%             end
    elseif i >= dimRow-l
%          for j=1:dimCol
            if j <= l+1
                A = Zpc((i-l):dimRow,j:(j+l),:);
            elseif j>= dimCol-l
                A = Zpc((i-l):dimRow,(j-l):dimCol,:);
            else
                A = Zpc((i-l):dimRow,(j-l):(j+l),:);
            end
%          end
    else 
%         for j=1:dimCol
            if j <= l+1
                A = Zpc((i-l):(i+l),j:(j+l),:);
            elseif j>= dimCol-l
                A = Zpc((i-l):(i+l),(j-l):dimCol,:);
            else
                A = Zpc((i-l):(i+l),(j-l):(j+l),:);
            end
%         end
    end
end




