%% Modified IQR method for each pixel

function [Zpc,Zol,ct] = modified_IQR(Zpc)

tic
[dimRow,dimCol,dimPage] = size(Zpc);
% Probabilities
p = [0.25 0.75];
% 'Whisker' for the modified IQR method
w = 1.5*(1+0.1*log(dimPage/10));
% Quantiles of the elements in data vector or array Zpc along the dimension 3
% for the probabilities p.
q = quantile(Zpc,p,3);
% Interquartile range along the dimension 3 of Zpc.
Y = iqr(Zpc,3);                                                            

% Definition of the modified IQR method to detect the outlines. 
Zol = Zpc < (q(:,:,1) - w.*Y) | ...                                        
    Zpc > (q(:,:,2) + w.*Y);
n_ol = sum(Zol,3);
% Identifies the non-outliers values using the modified IQR method.
Znol = Zpc > (q(:,:,1) - w.*Y) | ...                                       
    Zpc < (q(:,:,2) + w.*Y);
% The mean is performed taking into account only the non-outliers values.
M = mean(reshape(Zpc(Znol),size(Zpc)),3);                                  

figure('Name','Number of outliers per pixel-Modified IQR Method',...
'NumberTitle','off');
surf(1:dimRow,1:dimCol,n_ol,'LineStyle','none','FaceColor',...
    'flat','EdgeColor','flat');
colormap jet
title(['Total number of Outliers: ',num2str(sum(n_ol,'all'))],...
    'FontSize',14,'FontName','Times New Roman')
xlabel('pixels in x-direction','FontSize',12,'FontName','Times New Roman');
ylabel('pixels in y-direction','FontSize',12,'FontName','Times New Roman');


% The matrix with the mean value for each pixel is replicated along 
% the pages to have a matrix with the same size of Zpc.
M = repmat(M,[1 1 dimPage]);
% The outliers are replaced with the mean value. 
%The process is executed taking into account a single pixel each time.
Zpc(Zol) = M(Zol);                                                         
ct=toc;