%% Roughness parameters

% This function calculates the areal field parameters on the basis of the ISO 25178-2:2012.

function [Sa,Sq,Sdq] = roughness_parameters(Zpc,parameters,Ix,Iy)

[~,~,dimPage]=size(Zpc);
% Sa=[m]
Sa=zeros(1,dimPage);
% Sq=[m]
Sq=zeros(1,dimPage);
% Sdq=[rad]
Sdq=zeros(1,dimPage);

x_pixels=parameters(2); 
y_pixels=parameters(3);

parfor q = 1:dimPage
    Sa(q)=sum(sum(abs(Zpc(:,:,q))))./(x_pixels*y_pixels);
    Sq(q)=sqrt(sum(sum(Zpc(:,:,q).*Zpc(:,:,q)))./(x_pixels*y_pixels));
    
    % Zpc has to be expressed in nm because Ix and Iy are expressed in nm.
    [zdx,zdy]=gradient(Zpc(:,:,q)*1e3,Ix,Iy);                              
    Sdq(q)=sqrt(sum(zdx.^2+zdy.^2,'all')./(x_pixels*y_pixels));
end

